/***************************************************************************
                          bookmarkbutton.cpp  -  description
                             -------------------
    begin                : t j 8 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bookmarkbutton.h"
#include "pcmm_general.h"

#include <kiconloader.h>

BookmarkButton::BookmarkButton( QWidget *parent, const char *name )
    :QToolButton( /*Qt::DownArrow,*/ parent, name )
{
    bookmarks_menu = new QPopupMenu( this, "bookmarkspopupmenu" );

    KIconLoader *icon = kapp->iconLoader();
    setIconSet( icon->loadIconSet( "bookmark", KIcon::Toolbar ) );
    setTextLabel( i18n("Bookmarks") );
    setPopup( bookmarks_menu );
    setPopupDelay( 10 );

    directory_icon = icon->loadIcon( "folder_green", KIcon::Toolbar );
    home_icon = icon->loadIcon( "gohome", KIcon::Toolbar );

    connect( bookmarks_menu, SIGNAL( aboutToShow() ), this, SLOT( updateBookmarks() ) );
    connect( bookmarks_menu, SIGNAL( activated( int ) ), this, SLOT( gotoBookmark( int ) ) );
}

BookmarkButton::~BookmarkButton() {
}

void BookmarkButton::updateBookmarks() {
    bookmark_list = &Pcmm::url_database->bookmarkList();
    bookmarks_menu->clear();
    bookmarks_menu->insertItem( home_icon, i18n("Home") );
    QPtrListIterator<PURLItem> it( *bookmark_list );
    for ( ; it.current() ; ++it )
        bookmarks_menu->insertItem( directory_icon, (*it)->name() );
    bookmarks_menu->insertSeparator();
    bookmarks_menu->insertItem( i18n("&As other panel") );
    PCMM->configurebookmarks->plug( bookmarks_menu );
}

void BookmarkButton::gotoBookmark( int id ) {
    int index = bookmarks_menu->indexOf(id);
    if( index == 0 )
        emit listHome();
    else  if ((unsigned)index <= bookmark_list->count() ) {
        PURLItem *item =  bookmark_list->at(index-1);
        if (item)
            emit listBookmark( item->url().prettyURL() );
    }
    else if ((unsigned)index == bookmark_list->count()+2)
        slotListAsOtherPanel();
}

void BookmarkButton::slotListAsOtherPanel() {
    emit listAsOtherPanel();
}
