/***************************************************************************
                          generalimpl.cpp  -  description
                             -------------------
    begin                : Ne z 8 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "generalconfigimpl.h"
#include "../pcmm_general.h"

#include <kfiledialog.h>

GeneralConfigImpl::GeneralConfigImpl( QWidget *parent, const char *name )
: GeneralConfig( parent, name ) {
    setup();
}

GeneralConfigImpl::~GeneralConfigImpl() {
}

void GeneralConfigImpl::setup() {
    readSettings();
}

void GeneralConfigImpl::applyChanges() {
    saveSettings();
}

void GeneralConfigImpl::readSettings() {
    KConfig *config = kapp->config();
    config->setGroup( GENERAL_KEY );
    save_conf->setChecked( config->readBoolEntry( "Save configuration on exit", true ) );
    save_path->setChecked( config->readBoolEntry( "Save current path", true ) );
    editor->setText( config->readEntry( "Editor", "kwrite" ) );
    tempfile->setText( config->readEntry( "Temporary directory", "/tmp/pcmm.tmp" ) );
    database_file->setText( config->readEntry( "Database file", QString::null ) );
}

void GeneralConfigImpl::saveSettings() {
    KConfig *config = kapp->config();
    config->setGroup( GENERAL_KEY );
    config->writeEntry( "Save configuration on exit", save_conf->isChecked() );
    config->writeEntry( "Save current path", save_path->isChecked() );
    config->writeEntry( "Editor", editor->text() );
    config->writeEntry( "Temporary directory", tempfile->text() );
    config->writeEntry( "Database file", database_file->text() );
}

void GeneralConfigImpl::defaultSettings() {
    save_conf->setChecked( true );
    save_path->setChecked( true );
    editor->setText( "kwrite" );
    tempfile->setText( "/tmp/pcmm.tmp" );
    database_file->setText( QString::null );
}

void genericFileBrowse(QString startDir, QLineEdit *target, QString caption) {
    startDir=startDir.simplifyWhiteSpace();
    if (startDir.isEmpty()) startDir="/";
    QString dir=KFileDialog::getOpenFileName(startDir,0,0,caption);
    if (dir!=QString::null) target->setText(dir);
}

void genericDirBrowse(QString startDir, QLineEdit *target, QString caption) {
    startDir=startDir.simplifyWhiteSpace();
    if (startDir.isEmpty()) startDir="/";
    QString dir=KFileDialog::getExistingDirectory(startDir,0,caption);
    if (dir!=QString::null) target->setText(dir);
}

void GeneralConfigImpl::slotBrowseTempFile() {
    genericDirBrowse( tempfile->text(), tempfile, i18n("Choose a temporary directory"));    
}

void GeneralConfigImpl::slotBrowseDatabaseFile() {
    genericFileBrowse( database_file->text(), database_file, i18n("Choose a database file"));    
}

void GeneralConfigImpl::slotBrowseEditor() {
    genericFileBrowse( editor->text(), editor, i18n("Choose an editor") );
}
