#include "pdiskusagelabel.h"
#include <klocale.h>

#define INTERVAL 1000

PDiskUsageLabel::PDiskUsageLabel( QWidget *parent, const char *name )
: QLabel( parent, name ), timer( this ), working(false) {
}

PDiskUsageLabel::~PDiskUsageLabel() {
}

void PDiskUsageLabel::registerManager( PFileManager *manager_ ) {
    manager = manager_;
    
    connect( &timer, SIGNAL( timeout() ), SLOT( doTimer() ) );
    timer.start( INTERVAL );
}

void PDiskUsageLabel::doTimer() {
    if (working || !manager->CurrentDirectory().isLocalFile())
            return;
    KDiskFreeSp *dfs = KDiskFreeSp::findUsageInfo( manager->CurrentDirectory().path() );
    connect( dfs, SIGNAL( foundMountPoint ( const unsigned long&, const unsigned long&, const unsigned long&, const QString& ) ),
                    SLOT( showDiskUsage( const unsigned long&, const unsigned long&, const unsigned long&, const QString& ) ) );
    connect( dfs, SIGNAL( done() ), SLOT( done() ) );
    working = true;
}

void PDiskUsageLabel::showDiskUsage ( const unsigned long& , const unsigned long& , const unsigned long& MBAvail, const QString&  ) {
    QString text;
//	QTextOStream( &text ) << MBAvail << i18n( "MB free of " ) << MBSize << "MB";
    QTextOStream( &text ) << MBAvail << " MB";
    setText( text );
}

void PDiskUsageLabel::done() {
    working = false;
}

void PDiskUsageLabel::focusInParent () {
    setPaletteForegroundColor( palette().active().highlightedText() );

    update();	
}

void PDiskUsageLabel::focusOutParent () {
    unsetPalette();

    update();	
}
