/***************************************************************************
                          pfilepanel.cpp  -  description
                             -------------------
    begin                : Fri Mar 22 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "pfilepanel.h"
#include "pconfig/panelconfigimpl.h"
#include "pcmm_general.h"

#include <stdio.h>
#include <time.h>
#include <sys/stat.h>
#include <qpalette.h>
#include <kdialogbase.h>
#include <qlayout.h>
#include <qgroupbox.h>
#include <kurlcompletion.h>

PFilePanel::PFilePanel( bool is_left_, QWidget *parent, const char *name ) 
: QWidget(parent,name), vbox( new QVBoxLayout( this ) ), is_directory_line(true),
is_path_combobox( true ), is_info_line( true ), is_left(is_left_)
 {
    KConfig *config = CONFIG;
    config->setGroup(is_left ? LEFT_PANEL_KEY : RIGHT_PANEL_KEY);

    directory_line = new PDirectoryLine( this, "directoryline" );
    vbox->addWidget( directory_line );

    path_combobox = new KHistoryCombo( true, this, "pathcombobox" );
    path_combobox->setEditable( true );
    path_combobox->setFocusPolicy( QWidget::ClickFocus );
    KURLCompletion *completion = new KURLCompletion( KURLCompletion::DirCompletion );
    completion->setIgnoreCase( true );
    path_combobox->setCompletionObject( completion );
    vbox->addWidget( path_combobox );

    file_view = new PFileView( is_left, this, "filesview" );
    vbox->addWidget( file_view );
    file_view->setFocus();

    file_manager = new PFileManager( is_left, file_view, CONFIG->readEntry( "Current path", "/") );
    //zde by mela byt registrace tohoto manageru u file_view
    //ale proto, ze neznam manager druheho panelu, je tato registrace v DoublePanelu	
    
    connect( file_manager, SIGNAL( URLChanged( KURL& ) ), this, SLOT( ShowNewURL( KURL& ) ) );

    info_line = new PSelectedFilesInfo( this, "infoline" );
    vbox->addWidget( info_line );

    showDirectoryLine( config->readBoolEntry("Show directory_line",true) );
    showPathCombobox( config->readBoolEntry("Show path_combobox",false) );
    showInfoLine( config->readBoolEntry("Show info_line",true) );

    directory_line->DULabel()->registerManager( file_manager );

    connect( directory_line->URLLabel(), SIGNAL( rightClickedURL() ), SLOTS, SLOT( GoURL() ) );
    connect( directory_line->URLLabel(), SIGNAL( leftClickedURL( const QString& ) ), this, SLOT( ChangeDirectory( const QString& ) ) );
    connect( directory_line->bookmarkButton(), SIGNAL( listBookmark( const QString& ) ), this, SLOT( ChangeDirectory( const QString& ) ) );
    connect( directory_line->bookmarkButton(), SIGNAL( listAsOtherPanel() ), this, SLOT(listAsOtherPanel() ) );
    connect( directory_line->bookmarkButton(), SIGNAL( listHome() ), this, SLOT(listHome() ) );

    connect( path_combobox, SIGNAL( returnPressed( const QString& ) ), this, SLOT( ChangeDirectory( const QString& ) ) );
    connect( path_combobox, SIGNAL( activated( const QString& )), path_combobox, SLOT( addToHistory( const QString& )));

    connect( file_view, SIGNAL( mSelectionChanged() ), this, SLOT( slotSelectedFiles() ) );
    connect( file_view, SIGNAL( currentChanged( QListViewItem*) ), this, SLOT( slotCurrentFile(QListViewItem*) ) );
}

PFilePanel::~PFilePanel(){
    KConfig *config  = CONFIG;
    config->setGroup( GENERAL_KEY );
    bool save_conf = config->readBoolEntry("Save configuration on exit", true);
    bool save_path = config->readBoolEntry("Save current path", true );
    if (save_conf) saveConfig();
    if (save_path) saveCurrentPath();
    delete file_manager;
}

void PFilePanel::configureDlg() {
    KDialogBase *dlg = new KDialogBase( KDialogBase::Plain, i18n("Configure panel"), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, this, "configuredialog", true );
    QFrame *page = dlg->plainPage();

    QGridLayout *layout = new QGridLayout( page );

    QGroupBox *groupbox = new QGroupBox( page, "groupbox" );
    groupbox->setColumnLayout( 0, Qt::Vertical );
    groupbox->layout()->setSpacing( 0 );
    groupbox->layout()->setMargin( 0 );
    PanelConfigImpl *pc = new PanelConfigImpl( is_left, page/*groupbox*/, "panelconfigimpl" );
    layout->addWidget( groupbox, 0, 0 );

    if ( dlg->exec() )
	pc->applyChanges();
}

void PFilePanel::ShowNewURL( KURL& new_url) {
    directory_line->URLLabel()->setURL( new_url.prettyURL() );
    path_combobox->insertURL( new_url, 0 );
}

PFileView *PFilePanel::fileView() {
    return file_view;
}

PFileManager *PFilePanel::fileManager() {
    return file_manager;
}

PDirectoryLine *PFilePanel::directoryLine() {
    return directory_line;
}

void PFilePanel::showDirectoryLine( bool show ) {
    if( show == isDirectoryLine() ) return;

    if (show) {
	directory_line->show();
        is_directory_line = true;
    }
    else {
        directory_line->hide();
        is_directory_line = false;
    }
    emit directoryLineShown( show );
}

void PFilePanel::showPathCombobox( bool show ) {
    if( show == isPathCombobox() ) return;

    if (show) {
	path_combobox->show();
        is_path_combobox = true;
    }
    else {
        path_combobox->hide();
        is_path_combobox = false;
    }
    emit pathComboboxShown( show );
}

void PFilePanel::showInfoLine( bool show ) {
    if( show == isInfoLine() ) return;

    if (show) {
	info_line->show();
        is_info_line = true;
    }
    else {
	info_line->hide();
        is_info_line = false;
    }
    emit infoLineShown( show );
}

void PFilePanel::toggleDirectoryLine() {
    showDirectoryLine( !isDirectoryLine() );
}

void PFilePanel::togglePathCombobox() {
    showPathCombobox( !isPathCombobox() );
}

void PFilePanel::toggleInfoLine() {
    showInfoLine( !isInfoLine() );
}

bool PFilePanel::isDirectoryLine() {
    return is_directory_line;
}

bool PFilePanel::isPathCombobox() {
    return is_path_combobox;
}

bool PFilePanel::isInfoLine() {
    return is_info_line;
}

void PFilePanel::ChangeDirectory( const QString& dir ) {
	file_manager->ListURL( KURL(dir) );
}

void PFilePanel::listAsOtherPanel() {
    file_manager->listAsOtherPanel();
}

void PFilePanel::listHome() {
    ChangeDirectory( getenv("HOME") );
}

void PFilePanel::saveConfig() {
    KConfig *config = CONFIG;
    config->setGroup(is_left ? LEFT_PANEL_KEY : RIGHT_PANEL_KEY );
    config->writeEntry( "Show directory_line", isDirectoryLine() );
    config->writeEntry( "Show path_combobox", isPathCombobox() );
    config->writeEntry( "Show info_line", isInfoLine() );
    config->sync();
}
void PFilePanel::saveCurrentPath() {
    CONFIG->setGroup(is_left ? LEFT_PANEL_KEY : RIGHT_PANEL_KEY );
    CONFIG->writeEntry( "Current path", file_manager->CurrentDirectory().url() );
}

void PFilePanel::focusInEventChild() {
    emit CurrentPanel( this );
    directory_line->focusInParent();

    //show that other panel doesn't have focus
    PFilePanel *other_panel = PCMM->filePanel()->filePanel( !is_left );
    other_panel->directory_line->focusOutParent();
    other_panel->file_view->focusOutParent();
}

void PFilePanel::slotSelectedFiles() {
    info_line->selectedFiles( file_view->KFileView::selectedItems() );
}

void PFilePanel::slotCurrentFile(QListViewItem *item) {
    info_line->currentFile( ((PFileViewItem*)item)->fileInfo() );
}

