/***************************************************************************
                          pfileview.h  -  description
                             -------------------
    begin                : Mon May 6 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PFILEVIEW_H
#define PFILEVIEW_H

class KFileItem;
class QWidget;
class QKeyEvent;
class PFileManager;

#include <kmimetyperesolver.h>
#include <kfileview.h>
#include <kiconloader.h>
#include <kpopupmenu.h>
#include <kapp.h>

#include "plistview.h"
#include "pfilemanager.h"

/**
 * An item for the view, that has a reference to its corresponding
 * @ref KFileItem.
 */
class PFileViewItem : public PListViewItem {
public:
    PFileViewItem( QListView *parent, KFileItem *fi )
    : PListViewItem( parent ), inf( fi ) {
        if( !up_icon )
            up_icon = new QPixmap(kapp->iconLoader()->loadIcon( "up", KIcon::Small ));
        init();
    }

    ~PFileViewItem() {
	    inf->removeExtraData( listView() );
    }

    /**
     * @returns the corresponding @ref KFileItem
     */
    KFileItem *fileInfo() const {
	    return inf;
    }

    /**
     * Virtual function used for sorting
     * @returns key for sorting
     */
    virtual QString key( int /*column*/, bool /*ascending*/ ) const {
	    return m_key;
    }

    void setKey( const QString& key ) { m_key = key; }

    /**
     * Update connected @ref KFileItem item by file
     */
    void updateFile( KFileItem *file ) {
	    inf->assign(*file);
	    init();
    }
    
    QRect rect() const
    {
	    QRect r = listView()->itemRect(this);
	    return QRect( listView()->viewportToContents( r.topLeft() ), QSize( r.width(), r.height() ) );
    }

    /**
     * Set text to appropriate columns
     */
    void init();

    /**
     * Set selected if sel if true
     */
    virtual void setmSelected( bool sel );

private:
    KFileItem *inf;
    QString m_key;
    static QPixmap *up_icon;

private:
    class PFileViewItemPrivate;
    PFileViewItemPrivate *d;

};

/**
* @short A widget for viewing of files. 
*
* A widget for files viewing. Reimplemplemented
* from class KFileDetailView from KDE libraries
*/
class PFileView : public PListView, public KFileView {
    Q_OBJECT

    friend PFileManager;
    friend PFileViewCache;
    friend PFileViewItem;
public:
    PFileView( bool is_left_panel, QWidget *parent, const char *name);
    virtual ~PFileView();

    /**
     * Register managers to the widget
     * @param manager Manager that the widget asks for "static" actions like to make new directory
     * @param manager_other Manager that the widget ask only for copying
     */
    virtual void RegisterManager( PFileManager *manager, PFileManager *manager_other );

    virtual QWidget *widget() { return this; }
    virtual void clearView();
    virtual void setAutoUpdate( bool ) {}

    virtual void updateView( bool );
    virtual void updateView(const KFileItem*);
    virtual void removeItem( const KFileItem *);
    KFileItem *removeItem( const QString& );

    KFileItem *setSelected(const QString&, bool);
    virtual void setSelected(const KFileItem *, bool);
    virtual bool isSelected(const KFileItem *i) const;
    virtual void clearSelection();
    virtual void selectAll();
    virtual void invertSelection();

    KFileItem *setCurrentItem( const QString &filename );
    virtual void setCurrentItem( const KFileItem * );
    virtual KFileItem * currentFileItem() const;
    virtual KFileItem * firstFileItem() const;
    virtual KFileItem * nextItem( const KFileItem * ) const;
    virtual KFileItem * prevItem( const KFileItem * ) const;

    // implemented to get noticed about sorting changes (for sortingIndicator)
    virtual void setSorting( QDir::SortSpec );

    void ensureItemVisible( const KFileItem * );

    // for KMimeTypeResolver
    void mimeTypeDeterminationFinished();
    void determineIcon( PFileViewItem *item );
    QScrollView *scrollWidget() const { return (QScrollView*) this; }

    /**
     * show that this view doesn't have focus
     */
    virtual void focusOutParent();

    /**
     * @returns true if size column is shown
     */
    bool isSizeCol();
    /**
     * Similar as above */
    bool isDateCol();
    /**
     * Similar as above */
    bool isPermCol();
    /**
     * Similar as above */
    bool isOwnerCol();
    /**
     * Similar as above */
    bool isGroupCol();

    /**
     * Show size column if show is true
     */
    void showSizeCol( bool show = true );
    /**
     * Similar as above */
    void showDateCol( bool show = true );
    /**
     * Similar as above */
    void showPermCol( bool show = true );
    /**
     * Similar as above */
    void showOwnerCol( bool show = true );
    /**
     * Similar as above */
    void showGroupCol( bool show = true );

    /**
     * On every item in the view @ref PFileViewItem::init is called
     */
    void redisplay();

    static QString sortingKey( const QString& value, bool isDir, int sortSpec);
    static QString sortingKey( KIO::filesize_t value, bool isDir,int sortSpec);

public slots:
    /**
     * item was activated - pushed Enter or doubleclicked.
     * Emits signal FileEnter()
     */
    void FileEntered( QListViewItem* );

    /**
     * Same as above */
    void FileEntered();

    /**
     * Emits signal @ref CopyFile or @ref CopyFiles
     */
    void CopyFiles();

    /**
     * Emits signal @ref MoveFile or @ref MoveFiles
     */
    void MoveFiles();

    /**
     * Emits signal @ref DeleteFile or @ref DeleteFiles
     */
    void DeleteFiles();

    /**
     * Emits signal @ref RenameFile
     */
    void RenameFile();
    
    /**
     * Emits signal @ref FileProperties
     */
    void FileProperties();

    /**
     * Emits signal @ref LinkFiles
     */
    void LinkFiles();

    void CopyFilesClip();
    void CutFiles();
    void PasteFiles();

    void readConfig();
    void saveConfig();

signals:
    /**
     * Signal notice connected manager that file was activated
     */
    void FileEnter( KFileItem *file );

    /**
     * Signal asks connected manager to copy the file
     */
    void CopyFile( const KFileItem* file);

    /**
     * Similar as above */
    void CopyFiles( const KFileItemList* );
    /**
     * Similar as above */
    void MoveFile( const KFileItem* );
    /**
     * Similar as above */
    void MoveFiles( const KFileItemList* );
    /**
     * Similar as above */
    void DeleteFile( const KFileItem* );
    /**
     * Similar as above */
    void DeleteFiles( const KFileItemList* );
    /**
     * Similar as above */
    void RenameFile( const KFileItem* );
    /**
     * Similar as above */
    void FileProperties( KFileItem * );
    /**
     * Similar as above */
    void LinkFile( const KFileItem * );
    /**
     * Similar as above */
    void LinkFiles( const KFileItemList * );

protected:
    virtual void listingCompleted();
    virtual void keyPressEvent( QKeyEvent * );
    virtual void insertItem( KFileItem *i );

    int m_sortingCol;

    virtual void focusInEvent( QFocusEvent* );

    virtual void startDrag();
    virtual QDragObject *dragObject();
    virtual bool acceptDrag (QDropEvent* event) const;

private slots:
    void slotSortingChanged( int );
    void selected( QListViewItem *item );
    void slotActivate( QListViewItem *item );
    void highlighted( QListViewItem *item );
    void slotActivateMenu ( QListViewItem *item, const QPoint& pos );

    void showFileProperties( QListViewItem * );
    void showContextMenu( KListView *view, QListViewItem *item, const QPoint& p);

    void filesDropped( QDropEvent *event);

    void copyDroppedFiles();
    void moveDroppedFiles();
    void linkDroppedFiles();

    void openWithActivated( int id );

private:
    virtual void insertItem(QListViewItem *i) { PListView::insertItem(i); }
    virtual void setSorting(int i, bool b) { PListView::setSorting(i, b); }
    virtual void setSelected(QListViewItem *i, bool b) { PListView::setSelected(i, b); }

    inline PFileViewItem * viewItem( const KFileItem *item ) const {
	if ( item )
	    return (PFileViewItem *) item->extraData( this );
	return 0L;
    }

    void setSortingKey( PFileViewItem *item, const KFileItem *i );

    void updateFileMenu( KFileItem *file );

    PFileManager* manager;
    PFileManager* manager_other;
    
    KMimeTypeResolver<PFileViewItem,PFileView> *m_resolver;

    KPopupMenu *drop_menu;

    // pole sloupcu
    int *cols;

    class PFileViewPrivate;
    PFileViewPrivate *d;

protected:
    virtual void virtual_hook( int id, void* data );
};

#endif // PFILEVIEW_H
