#include "pfileviewcache.h"

PFileViewCache::PFileViewCache( PFileView *view_, bool want_cache )
: current_file(0), cache_type(NONE), timer( this, "timer" ), view(view_),
use_cache(want_cache), clear_view_on_insert( false ), last_found(0) {
	if (use_cache)
		timer.start( INTERVAL );
		
        connect( &timer, SIGNAL( timeout() ), this, SLOT( TimerDo() ));
}

PFileViewCache::~PFileViewCache() {
}

void PFileViewCache::DeleteItem( KFileItem* file ) {
	//kdDebug(1) << "deleteitem" << endl;
	if (use_cache) {
		timer.start( INTERVAL );
		ClearInserted();
		cache.append( file );
		cache_type = DELETED;
	}
	else {
		view->removeItem( file );
//		delete file;
	}
	last_found = 0;
}

void PFileViewCache::DeleteItem( QString file ) {
	//kdDebug(1) << "deleteitem2" << endl;
	const KFileItemList *file_list = view->items();
	KFileItemListIterator it(*file_list);
	for( ; it.current() ; ++it )
		if ( (*it)->name() == file ) {
			DeleteItem( *it );
			return;
		}
	if ( cache_type == INSERTED ) {
		KFileItemListIterator it(cache);
		for( ; it.current() ; ++it )
			if ( (*it)->name() == file ) {
				if (last_found && last_found->name() == (*it)->name() ) last_found = 0;
				if (current_file && current_file->name() == (*it)->name() ) current_file = 0;
				cache.setAutoDelete(true);
				cache.removeRef(*it);
				cache.setAutoDelete(false);
				if ( cache.isEmpty() ) cache_type = NONE;
				return;
			}
	}
}

void PFileViewCache::InsertItem( KFileItem *file, bool set_current, bool clear_view ) {
//	kdDebug(1) << "insertitem" << endl;
	if (clear_view) { //pri prvnim vyprazdnovani cache se vymaze i view
		clear_view_on_insert = true;
		DeleteCache();
	}
	if (set_current) //pro prvnim vyprazdnovani cache se nastavi jako aktualni
		current_file = file;
	if (use_cache) {
		timer.start( INTERVAL );
		ClearDeleted();
		cache_type = PFileViewCache::INSERTED;
		cache.append( file );
	}
	else {
		if (clear_view_on_insert) ClearView();
		view->insertItem( file );
                if (set_current) SetFileCurrent();
	}
}

void PFileViewCache::UpdateItem( KFileItem *update, KFileItem *by ) {
	ClearCache();

	// WARNNG update must be shown in view => to provide extraData()
	PFileViewItem *item = (PFileViewItem*)update->extraData(view);
	if (!item) return;
	item->updateFile( by );
	view->setSortingKey( item, update );
}

void PFileViewCache::UpdateItem( KFileItem *file ) {
	//kdDebug(1) << "...updating... " << file->name() << endl;
	ClearDeleted();

	//pokus se najit v tomto adresari file
	KFileItem *found = SearchItem( file );

	//jestli existuje, tak ho updatni
	if ( found ) {
		kdDebug(1) << "...updating... Updatuju soubor " << found->name() << endl;
		UpdateItem( found, file );
		delete file;
	}
	//jestli jeste neexistuje, tak ho vytvor
	else {
		kdDebug(1) << "...updating... Vytvarim soubor " << file->name() << endl;
		InsertItem( file );
	}
}

void PFileViewCache::setSelected( KFileItem *file, bool sel ) {
	ClearCache();

	view->setSelected( file, sel );
}

void PFileViewCache::setSelected( QString file, bool sel ) {
	const KFileItemList *file_list = view->items();
	KFileItemListIterator it(*file_list);
	for( ; it.current() ; ++it )
		if ( (*it)->name() == file ) {
			setSelected( *it, sel );
			return;
		}
}

void PFileViewCache::UpdateView( KFileItemList& files_to_add, KFileItemList& files_to_remove, KFileItemList& files_to_update ) {
	ClearCache();

	//pridat nove soubory
	view->addItemList( files_to_add );

	//odebrat stare soubory
	KFileItemListIterator it( files_to_remove );
	for( ; it.current(); ++it )
		view->removeItem( *it );

	//updatovat zmenene soubory
	KFileItemListIterator it_up( files_to_update );
	for( ; it_up.current() ; ++it_up )
		UpdateItem( (KFileItem*)(*it_up)->extraData( this ), *it_up );

	//opravdove smazu soubory uz zrusenych souboru a souboru k updatovani 
	// (podle nich se updatuje, ted jsou na nic)
//	files_to_remove.setAutoDelete( true );
	files_to_update.setAutoDelete( true );
	files_to_remove.clear();
	files_to_update.clear();
//	files_to_remove.setAutoDelete( false );
	files_to_update.setAutoDelete( false );

	last_found = 0;
}

void PFileViewCache::InsertToView( KFileItemList& files_to_insert, QString new_current_file ) {
	ClearCache();

	//zrusi stare soubory
	KFileItemList list( *view->items() );
	list.setAutoDelete(true);
	view->KFileView::clear();
	list.clear();
	
	view->addItemList( files_to_insert );
	if (!new_current_file.isNull()) {
		view->ensureItemVisible( view->setCurrentItem( new_current_file ) );
	}
	else
		view->setCurrentItem( view->firstFileItem() );

	last_found = 0;
}

KFileItemList& PFileViewCache::items( bool clear_cache ) {
	//kdDebug(1) << "items" << endl;
	if (clear_cache) ClearCache();
	m_items = *view->items();
	if (use_cache && cache_type != NONE) {
		if (cache_type == PFileViewCache::DELETED) {
			//odeberu soubory, ktere jsou v cache
			KFileItemListIterator it(cache);
			for( ; it.current() ; ++it )
				m_items.removeRef( *it );
		}
		else { // cache_type == PFileViewCache::INSERTED
			//pridam soubory, ktere jsou v cache
			KFileItemListIterator it(cache);
			for( ; it.current() ; ++it )
				m_items.append( *it );
		}
	}
	return m_items;
}

void PFileViewCache::ClearCache() {
	if ( cache_type != NONE ) {
		if (cache_type == PFileViewCache::DELETED)
			ClearDeleted();
		else
			ClearInserted();
	}
}

void PFileViewCache::DeleteCache() {
//	kdDebug(1) << "deletecache" << endl;
	if ( cache_type != NONE ) {
		if (cache_type == PFileViewCache::DELETED)
			cache.clear();
		else {
			cache.setAutoDelete( true );
			cache.clear();
			cache.setAutoDelete( false );
			last_found = 0;
		}
		cache_type = NONE;
	}
}

void PFileViewCache::ClearView() {
	//kdDebug(1) << "clearview" << endl;
	clear_view_on_insert = false;

	//zrusi stare soubory
	KFileItemList list( *view->items() );
	view->KFileView::clear();
	list.setAutoDelete(true);
	list.clear();

	last_found = 0;
}

void PFileViewCache::setCache( bool want_cache ) {
    if (want_cache == use_cache) return;
    if (!want_cache) {
	ClearCache();
	timer.stop();
    }
    else
	timer.start( INTERVAL );

    use_cache = want_cache;
}

bool PFileViewCache::hasCache() {
    return use_cache;
}

void PFileViewCache::ClearDeleted() {
	if ( cache_type == DELETED) {
		//kdDebug(1) << "cleardeleted" << endl;
		KFileItemListIterator it( cache );
		for( ; it.current() ; ++it ) {
			view->removeItem( *it );
		}
//		cache.setAutoDelete( true );
		cache.clear();
//		cache.setAutoDelete( false );
		cache_type = NONE;

		last_found = 0;
	}
}

void PFileViewCache::ClearInserted() {
	if ( clear_view_on_insert ) ClearView();
	if ( cache_type == INSERTED) {
		//kdDebug(1) << "clearinserted" << endl;
		KFileItemListIterator it( cache );
		view->addItemList( cache );
		cache.clear();
		cache_type = NONE;
	}
	if ( current_file ) SetFileCurrent();
}

KFileItem *PFileViewCache::SearchItem( KFileItem *file ) {
	//kdDebug(1) << "searchitem" << endl;
	if ( last_found && last_found->url() == file->url() )
		return last_found;
	else {
		KFileItemListIterator it( *view->items() );
		for ( ; it.current() ; ++it )
			if ((*it)->name() == file->name())
				return last_found = *it;

		if (cache_type == INSERTED) {
			KFileItemListIterator it_cache( cache );
			for ( ; it_cache.current() ; ++it_cache )
				if ((*it_cache)->name() == file->name())
					return last_found = *it_cache;
		}
		return last_found = 0;
	}
}

void PFileViewCache::TimerDo() {
//	kdDebug(1) << "timerdo" << endl;
	ClearCache();
}

//! pouze kdyz uz je ve view !!!!
void PFileViewCache::SetFileCurrent() {
    //kdDebug(1) << "setfilecurrent" << endl;
    if (current_file && current_file->extraData(view) ) {
	view->setCurrentItem( current_file );
	view->ensureItemVisible( current_file );
    }
    current_file = 0;
}
