#include <klocale.h>
#include <pwd.h>
#include <grp.h>
#include <sys/types.h>

#include "advanced.h"
#include "pcheckbox.h"

#include <qvariant.h>
#include <kcombobox.h>
#include <krestrictedline.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <kmessagebox.h>

/* 
 *  Constructs a Advanced which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 */
Advanced::Advanced( FinderQuery *query_, QWidget* parent,  const char* name )
    : KDialogBase( Plain, i18n("Advanced"), Ok|Cancel|Default, Ok, parent, name ), query( query_ )
{
    if ( !name )
	setName( "Advanced" );
    resize( 1, 1 ); 
    setMaximumSize( QSize( 0, 0 ) );

    QFrame *main_page = plainPage();
    AdvancedLayout = new QGridLayout( main_page, 1, 1, 0, 6, "AdvancedLayout"); 

    QHBoxLayout *line_layout1 = new QHBoxLayout( 0, 0, 6, "line_layout1" );
    QLabel *line_label1 = new QLabel( i18n("Permissions" ), main_page );
    QFrame *line1 = new QFrame( main_page, "line1" );
    line1->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    line_layout1->addWidget( line_label1 );
    line_layout1->addWidget( line1 );
    line_layout1->setStretchFactor( line1, 1 );
    AdvancedLayout->addMultiCellLayout( line_layout1, 0, 0, 0, 4 );

    Layout1 = new QHBoxLayout( 0, 0, 6, "Layout1"); 

  /*  owner_label = new QLabel( main_page, "owner_label" );
    owner_label->setText( tr2i18n( "Owner" ) );
    Layout1->addWidget( owner_label );*/

    owner_combo = new KComboBox( FALSE, main_page, "owner_combo" );
    owner_combo->insertItem( tr2i18n( "- owner -" ) );
    struct passwd *ps;
    while( ps = getpwent() )
        owner_combo->insertItem( ps->pw_name );
    endpwent();
    Layout1->addWidget( owner_combo );

    AdvancedLayout->addMultiCellLayout( Layout1, 1, 1, 0, 1 );

    Layout1_3 = new QHBoxLayout( 0, 0, 6, "Layout1_3"); 

/*    group_label = new QLabel( main_page, "group_label" );
    group_label->setText( tr2i18n( "Group" ) );
    Layout1_3->addWidget( group_label );*/

    group_combo = new KComboBox( FALSE, main_page, "group_combo" );
    group_combo->insertItem( tr2i18n( "- group -" ) );
    struct group *gr;
    while( gr = getgrent() )
        group_combo->insertItem( gr->gr_name );
    endgrent();
    Layout1_3->addWidget( group_combo );

    AdvancedLayout->addLayout( Layout1_3, 1, 2 );

    others_label = new QLabel( main_page, "others_label" );
    others_label->setText( tr2i18n( "Others" ) );

    AdvancedLayout->addWidget( others_label, 1, 3 );

    special_label = new QLabel( main_page, "special_label" );
    special_label->setText( tr2i18n( "Special" ) );

    AdvancedLayout->addWidget( special_label, 1, 4 );

    owner_read = new PCheckBox( main_page, "owner_read" );
    owner_read->setText( tr2i18n( "Read" ) );

    AdvancedLayout->addMultiCellWidget( owner_read, 2, 2, 0, 1 );

    group_read = new PCheckBox( main_page, "group_read" );
    group_read->setText( tr2i18n( "Read" ) );

    AdvancedLayout->addWidget( group_read, 2, 2 );

    group_write = new PCheckBox( main_page, "group_write" );
    group_write->setText( tr2i18n( "Write" ) );

    AdvancedLayout->addWidget( group_write, 3, 2 );

    group_exec = new PCheckBox( main_page, "group_exec" );
    group_exec->setText( tr2i18n( "Exec" ) );

    AdvancedLayout->addWidget( group_exec, 4, 2 );

    others_read = new PCheckBox( main_page, "others_read" );
    others_read->setText( tr2i18n( "Read" ) );

    AdvancedLayout->addWidget( others_read, 2, 3 );

    others_write = new PCheckBox( main_page, "others_write" );
    others_write->setText( tr2i18n( "Write" ) );

    AdvancedLayout->addWidget( others_write, 3, 3 );

    others_exec = new PCheckBox( main_page, "others_exec" );
    others_exec->setText( tr2i18n( "Exec" ) );

    AdvancedLayout->addWidget( others_exec, 4, 3 );

    sticky = new PCheckBox( main_page, "sticky" );
    sticky->setText( tr2i18n( "Sticky" ) );

    AdvancedLayout->addWidget( sticky, 4, 4 );

    gid = new PCheckBox( main_page, "gid" );
    gid->setText( tr2i18n( "Set GID" ) );

    AdvancedLayout->addWidget( gid, 3, 4 );

    uid = new PCheckBox( main_page, "uid" );
    uid->setText( tr2i18n( "Set UID" ) );

    AdvancedLayout->addWidget( uid, 2, 4 );

    owner_write = new PCheckBox( main_page, "owner_write" );
    owner_write->setText( tr2i18n( "Write" ) );

    AdvancedLayout->addMultiCellWidget( owner_write, 3, 3, 0, 1 );

    owner_exec = new PCheckBox( main_page, "owner_exec" );
    owner_exec->setText( tr2i18n( "Exec" ) );

    AdvancedLayout->addMultiCellWidget( owner_exec, 4, 4, 0, 1 );

    QHBoxLayout *line_layout2 = new QHBoxLayout( 0, 0, 6, "line_layout2" );
    QLabel *line_label2 = new QLabel( i18n("Type" ), main_page );
    QFrame *line2 = new QFrame( main_page, "line2" );
    line2->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    line_layout2->addWidget( line_label2 );
    line_layout2->addWidget( line2 );
    line_layout2->setStretchFactor( line2, 1 );
    AdvancedLayout->addMultiCellLayout( line_layout2, 5, 5, 0, 4 );

    regular_file = new PCheckBox( main_page, "regular_file" );
    regular_file->setText( tr2i18n( "Regular File" ) );

    AdvancedLayout->addMultiCellWidget( regular_file, 6, 6, 0, 1 );

    directory = new PCheckBox( main_page, "directory" );
    directory->setText( tr2i18n( "Directory" ) );

    AdvancedLayout->addMultiCellWidget( directory, 7, 7, 0, 1 );

    block_device = new PCheckBox( main_page, "block_device" );
    block_device->setText( tr2i18n( "Block Device" ) );

    AdvancedLayout->addWidget( block_device, 7, 2 );

    character_device = new PCheckBox( main_page, "character_device" );
    character_device->setText( tr2i18n( "Character Device" ) );

    AdvancedLayout->addWidget( character_device, 6, 2 );

    symbolic_link = new PCheckBox( main_page, "symbolic_link" );
    symbolic_link->setText( tr2i18n( "Symbolic Link" ) );

    AdvancedLayout->addWidget( symbolic_link, 6, 3 );

    fifo = new PCheckBox( main_page, "fifo" );
    fifo->setText( tr2i18n( "Fifo" ) );

    AdvancedLayout->addWidget( fifo, 7, 3 );

    socket = new PCheckBox( main_page, "socket" );
    socket->setText( tr2i18n( "Socket" ) );

    AdvancedLayout->addWidget( socket, 7, 4 );

    QHBoxLayout *line_layout3 = new QHBoxLayout( 0, 0, 6, "line_layout3" );
    QLabel *line_label3 = new QLabel( i18n("Size, Date and Time" ), main_page );
    QFrame *line3 = new QFrame( main_page, "line3" );
    line3->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    line_layout3->addWidget( line_label3 );
    line_layout3->addWidget( line3 );
    line_layout3->setStretchFactor( line3, 1 );
    AdvancedLayout->addMultiCellLayout( line_layout3, 8, 8, 0, 4 );

    size_label = new QLabel( main_page, "size_label" );
    size_label->setText( tr2i18n( "Size" ) );

    AdvancedLayout->addWidget( size_label, 9, 0 );

    date_label = new QLabel( main_page, "date_label" );
    date_label->setText( tr2i18n( "Date" ) );

    AdvancedLayout->addWidget( date_label, 10, 0 );

    time_label = new QLabel( main_page, "time_label" );
    time_label->setText( tr2i18n( "Time" ) );

    AdvancedLayout->addWidget( time_label, 11, 0 );

    date_format_label = new QLabel( main_page, "date_format_label" );
    date_format_label->setEnabled( FALSE );
    date_format_label->setText( tr2i18n( "dd.mm.yyyy" ) );

    AdvancedLayout->addWidget( date_format_label, 10, 4 );

    time_format_label = new QLabel( main_page, "time_format_label" );
    time_format_label->setEnabled( FALSE );
    time_format_label->setText( tr2i18n( "hh:mm:ss" ) );

    AdvancedLayout->addWidget( time_format_label, 11, 4 );

    size_combo = new KComboBox( FALSE, main_page, "size_combo" );
    size_combo->insertItem( tr2i18n( "not used" ) );
    size_combo->insertItem( tr2i18n( "greater than" ) );
    size_combo->insertItem( tr2i18n( "greater or equal to" ) );
    size_combo->insertItem( tr2i18n( "equal to" ) );
    size_combo->insertItem( tr2i18n( "smaller or equal to" ) );
    size_combo->insertItem( tr2i18n( "smaller than" ) );

    AdvancedLayout->addMultiCellWidget( size_combo, 9, 9, 1, 2 );

    date_input = new KRestrictedLine( main_page, "date_input" );
    date_input->setEnabled( FALSE );
    date_input->setAlignment( Qt::AlignRight );
    date_input->setValidChars( tr2i18n( "0123456789." ) );

    AdvancedLayout->addWidget( date_input, 10, 3 );

    date_combo = new KComboBox( FALSE, main_page, "date_combo" );
    date_combo->insertItem( tr2i18n( "not used" ) );
    date_combo->insertItem( tr2i18n( "older than" ) );
    date_combo->insertItem( tr2i18n( "older or equal to" ) );
    date_combo->insertItem( tr2i18n( "equal to" ) );
    date_combo->insertItem( tr2i18n( "newer or equal to" ) );
    date_combo->insertItem( tr2i18n( "newer than" ) );

    AdvancedLayout->addMultiCellWidget( date_combo, 10, 10, 1, 2 );

    time_combo = new KComboBox( FALSE, main_page, "time_combo" );
    time_combo->insertItem( tr2i18n( "not used" ) );
    time_combo->insertItem( tr2i18n( "used" ) );
    time_combo->setEnabled( FALSE );

    AdvancedLayout->addMultiCellWidget( time_combo, 11, 11, 1, 2 );

    time_input = new KRestrictedLine( main_page, "time_input" );
    time_input->setEnabled( FALSE );
    time_input->setAlignment( Qt::AlignRight );
    time_input->setValidChars( tr2i18n( "0123456789:" ) );

    AdvancedLayout->addWidget( time_input, 11, 3 );

    size_input = new KRestrictedLine( main_page, "size_input" );
    size_input->setEnabled( FALSE );
    size_input->setAlignment( Qt::AlignRight );
    size_input->setValidChars( tr2i18n( "0123456789" ) );

    AdvancedLayout->addWidget( size_input, 9, 3 );

    size_format_label = new QLabel( main_page, "size_format_label" );
    size_format_label->setEnabled( FALSE );
    size_format_label->setText( tr2i18n( "bytes" ) );

    AdvancedLayout->addWidget( size_format_label, 9, 4 );

    QHBoxLayout *line_layout4 = new QHBoxLayout( 0, 0, 6, "line_layout4" );
    QLabel *line_label4 = new QLabel( i18n("Options" ), main_page );
    QFrame *line4 = new QFrame( main_page, "line4" );
    line4->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    line_layout4->addWidget( line_label4 );
    line_layout4->addWidget( line4 );
    line_layout4->setStretchFactor( line4, 1 );
    AdvancedLayout->addMultiCellLayout( line_layout4, 12, 12, 0, 4 );

    complementary_names = new QCheckBox( main_page, "complementary_names" );
    complementary_names->setText( tr2i18n( "Search for complementary names" ) );
    AdvancedLayout->addMultiCellWidget( complementary_names, 13, 13, 0, 4 );

    QFrame *line5 = new QFrame( main_page, "line5" );
    line5->setFrameStyle( QFrame::HLine | QFrame::Sunken );
    AdvancedLayout->addMultiCellWidget( line5, 14, 14, 0, 4 );
    
    // signals and slots connections
    connect( date_combo, SIGNAL( activated(int) ), this, SLOT( setDateInputActivated(int) ) );
    connect( time_combo, SIGNAL( activated(int) ), this, SLOT( setTimeInputActivated(int) ) );
    connect( size_combo, SIGNAL( activated(int) ), this, SLOT( setSizeInputActivated(int) ) );
    connect( this, SIGNAL( okClicked() ), this, SLOT( saveToQuery() ) );

    setAsQuery();
}

/*  
 *  Destroys the object and frees any allocated resources
 */
Advanced::~Advanced()
{
    // no need to delete child widgets, Qt does it all for us
}

void Advanced::setTimeInputActivated( int i ) {
    time_input->setEnabled( i );
    time_format_label->setEnabled( i );
}

void Advanced::setDateInputActivated( int i ) {
    date_input->setEnabled( i );
    date_format_label->setEnabled( i );

    time_combo->setEnabled(i);
    if (!i) time_combo->setCurrentItem( 0 );
    setTimeInputActivated( 0 );
}

void Advanced::setSizeInputActivated( int i ) {
    size_input->setEnabled( i );
    size_format_label->setEnabled( i );
}

void Advanced::saveToQuery() {
    if ( owner_combo->currentItem() ) query->owner = owner_combo->currentText();
    else query->owner = QString::null;
    if ( group_combo->currentItem() ) query->group = group_combo->currentText();
    else query->group = QString::null;
    query->owner_read = owner_read->state();
    query->owner_write = owner_write->state();
    query->owner_exec = owner_exec->state();
    query->group_read = group_read->state();
    query->group_write = group_write->state();
    query->group_exec = group_exec->state();
    query->others_read = others_read->state();
    query->others_write = others_write->state();
    query->others_exec = others_exec->state();
    query->suid = uid->state();
    query->sgid = gid->state();
    query->sticky = sticky->state();
    query->t_regular = regular_file->state();
    query->t_dir = directory->state();
    query->t_char = character_device->state();
    query->t_block = block_device->state();
    query->t_symlink = symbolic_link->state();
    query->t_fifo = fifo->state();
    query->t_socket = socket->state();
    query->size_type = size_combo->currentItem();
    query->size = size_input->text().toULong();
    query->date_type = date_combo->currentItem() ? 6-date_combo->currentItem() : 0;
    query->complementary_names = complementary_names->isChecked();
    struct tm tm;
    tm.tm_mday = 0;
    tm.tm_sec = 0;
    tm.tm_mon = 0;
    tm.tm_year = 0;
    tm.tm_hour = 0;
    tm.tm_min = 0;
    tm.tm_isdst = 0;
    if (date_combo->currentItem()) {
        QString text = date_input->text();
        tm.tm_mday = text.section( '.', 0, 0 ).toInt();
        tm.tm_mon = text.section( '.', 1, 1 ).toInt();
        tm.tm_year = text.section( '.', 2, 2 ).toInt()-1900;
    }
    else {
        query->time = 0;
        return;
    }
    if (time_combo->currentItem()) {
        QString text = time_input->text();
        tm.tm_hour = text.section( ':', 0, 0 ).toInt();
        tm.tm_min = text.section( ':', 1, 1 ).toInt();
        tm.tm_sec = text.section( ':', 2, 2 ).toInt();
    }
    time_t t = mktime( &tm );
    if (t == (time_t)-1) {
        KMessageBox::sorry( this, i18n("You have entered invalid date or time") );
        query->date_type = FinderQuery::NotUsed;
        t = 0;
    }
    query->time = t;
}

void Advanced::setAsQuery() {
    if ( !query->owner.isNull() )
        for( int index = 0 ; index < owner_combo->count() ; index++ )
            if (query->owner == owner_combo->text( index ) ) {
                owner_combo->setCurrentItem( index );
                break;
            }
    if ( !query->group.isNull() )
        for( int index = 0 ; index < group_combo->count() ; index++ )
            if (query->group == group_combo->text( index ) ) {
                group_combo->setCurrentItem( index );
                break;
            }
    if ( query->owner_read != FinderQuery::NotUsedPerm )
        owner_read->setChecked( query->owner_read );
    if ( query->owner_write != FinderQuery::NotUsedPerm )
        owner_write->setChecked( query->owner_write );
    if ( query->owner_exec != FinderQuery::NotUsedPerm )
        owner_exec->setChecked( query->owner_exec );
    if ( query->group_read != FinderQuery::NotUsedPerm )
        group_read->setChecked( query->group_read );
    if ( query->group_write != FinderQuery::NotUsedPerm )
        group_write->setChecked( query->group_write );
    if ( query->group_exec != FinderQuery::NotUsedPerm )
        group_exec->setChecked( query->group_exec );
    if ( query->others_read != FinderQuery::NotUsedPerm )
        others_read->setChecked( query->others_read );
    if ( query->others_write != FinderQuery::NotUsedPerm )
        others_write->setChecked( query->others_write );
    if ( query->others_exec != FinderQuery::NotUsedPerm )
        others_exec->setChecked( query->others_exec );
    if ( query->suid != FinderQuery::NotUsedPerm )
        uid->setChecked( query->suid );
    if ( query->sgid != FinderQuery::NotUsedPerm )
        gid->setChecked( query->sgid );
    if ( query->sticky != FinderQuery::NotUsedPerm )
        sticky->setChecked( query->sticky );
    if ( query->t_regular != FinderQuery::NotUsedPerm )
        regular_file->setChecked( query->t_regular );
    if ( query->t_dir != FinderQuery::NotUsedPerm )
        directory->setChecked( query->t_dir );
    if ( query->t_char != FinderQuery::NotUsedPerm )
        character_device->setChecked( query->t_char );
    if ( query->t_block != FinderQuery::NotUsedPerm )
        block_device->setChecked( query->t_block );
    if ( query->t_symlink != FinderQuery::NotUsedPerm )
        symbolic_link->setChecked( query->t_symlink );
    if ( query->t_fifo != FinderQuery::NotUsedPerm )
        fifo->setChecked( query->t_fifo );
    if ( query->t_socket != FinderQuery::NotUsedPerm )
        socket->setChecked( query->t_socket );
    size_combo->setCurrentItem( query->size_type );
    size_input->setEnabled(  size_combo->currentItem() );
    size_format_label->setEnabled( size_combo->currentItem() );
    date_combo->setCurrentItem( query->date_type ? 6-query->date_type : 0 );
    date_input->setEnabled( date_combo->currentItem() );
    date_format_label->setEnabled( date_combo->currentItem() );
    time_combo->setEnabled( date_combo->currentItem() );
    size_input->setText( QString::number(query->size) );
    time_t m_time = query->time;
    if (m_time == 0) time( &m_time );
    struct tm tm;
    localtime_r( &m_time, &tm );
    date_input->setText( QString("%1.%2.%3").arg(tm.tm_mday).arg(tm.tm_mon).arg(tm.tm_year+1900) );
    time_input->setText( QString("%1:%2:%3").arg(tm.tm_hour).arg(tm.tm_min).arg(tm.tm_sec) );
    if (tm.tm_hour && tm.tm_min && tm.tm_sec && date_combo->currentItem() ) time_combo->setCurrentItem( 1 );
    else time_combo->setCurrentItem( 0 );
    time_input->setEnabled(  time_combo->currentItem() );
    time_format_label->setEnabled(  time_combo->currentItem() );

    complementary_names->setChecked(query->complementary_names);
}

void Advanced::slotDefault() {
    owner_combo->setCurrentItem(0);
    group_combo->setCurrentItem(0);
    owner_read->setNoChange();
    group_read->setNoChange();
    group_write->setNoChange();
    group_exec->setNoChange();
    others_read->setNoChange();
    others_write->setNoChange();
    others_exec->setNoChange();
    sticky->setNoChange();
    gid->setNoChange();
    uid->setNoChange();
    owner_write->setNoChange();
    owner_exec->setNoChange();
    regular_file->setNoChange();
    directory->setNoChange();
    block_device->setNoChange();
    character_device->setNoChange();
    symbolic_link->setNoChange();
    fifo->setNoChange();
    socket->setNoChange();
    size_combo->setCurrentItem(0);
    date_combo->setCurrentItem(0);
    time_combo->setCurrentItem(0);
    size_combo->setEnabled(true);
    date_combo->setEnabled(true);
    time_combo->setEnabled(false);
    size_input->setText(0);
    time_t m_time;
    time( &m_time );
    struct tm tm;
    localtime_r( &m_time, &tm );
    date_input->setText( QString("%1.%2.%3").arg(tm.tm_mday).arg(tm.tm_mon).arg(tm.tm_year+1900) );
    time_input->setText( QString("%1:%2:%3").arg(tm.tm_hour).arg(tm.tm_min).arg(tm.tm_sec) );
    size_input->setEnabled(false);
    date_input->setEnabled(false);
    time_input->setEnabled(false);
    size_format_label->setEnabled(false);
    date_format_label->setEnabled(false);
    time_format_label->setEnabled(false);
    complementary_names->setChecked(false);
}


#include "advanced.moc"
