/***************************************************************************
                          plistjob.h  -  description
                             -------------------
    begin                : P j 4 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LISTJOB_H
#define LISTJOB_H

#include <kio/job.h>
#include <kio/jobclasses.h>

using namespace KIO;

class PListJob;

class RecursiveAllocator: public QObject {
    Q_OBJECT
    friend PListJob;
signals:
    /**
     * Signals currently listed directory
     */
    void currentDir( const KURL& url );
private slots:
    void emitCurrentDir( const KURL& url ) {
        emit currentDir( url );
    };
};

/**
 * This class is modified @ref KIO::ListJob
 */
class PListJob: public SimpleJob {
    Q_OBJECT
public:
    PListJob( RecursiveAllocator *, const KURL& url, bool showProgressInfo,
            bool recursive = false, QString prefix = QString::null,
            bool includeHidden = true);

    virtual void start( Slave *slave );

    static PListJob *listDir( RecursiveAllocator*, const KURL& url, bool showProgressInfo = true, bool includeHidden = true );
    static PListJob *listRecursive( RecursiveAllocator*, const KURL& url, bool showProgressInfo = true, bool includeHidden = true );

signals:
    /**
     * This signal emits the entry found by the job while listing.
     * The progress signals aren't specific to ListJob. It simply
     * uses SimpleJob's @ref processedSize (number of entries listed) and
     * @ref totalSize (total number of entries, if known),
     * as well as percent.
     */
    void entries( KIO::Job *, const KIO::UDSEntryList& );

    /**
     * Signals a redirection.
     * Use to update the URL shown to the user.
     * The redirection itself is handled internally.
     */
    void redirection( KIO::Job *, const KURL &url );

protected slots:
    virtual void slotFinished( );
    virtual void slotResult( KIO::Job *job );
    void slotListEntries( const KIO::UDSEntryList& list );
    void slotRedirection( const KURL &url );
    void gotEntries( KIO::Job * subjob, const KIO::UDSEntryList& list );

private:
    bool recursive;
    bool includeHidden;
    QString prefix;
    unsigned long m_processedEntries;
    KURL m_redirectionURL;

    RecursiveAllocator *allocator;

protected:
    virtual void virtual_hook( int id, void* data );
private:
    class ListJobPrivate* d;
};

#endif
