/***************************************************************************
                          query.cpp  -  description
                             -------------------
    begin                : Ne z 29 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "query.h"

FinderQuery::FinderQuery()
    :pattern_case_sensitive( true ),
    pattern_regular_expression( false ),
    pattern_part_match( false ),
    search_subdir( true ),
    search_content( false ),
    content_case_sensitive( true ),
    content_regular_expression( false ),
    owner_read( NotUsedPerm ),
    owner_write( NotUsedPerm ),
    owner_exec( NotUsedPerm ),
    group_read( NotUsedPerm ),
    group_write( NotUsedPerm ),
    group_exec( NotUsedPerm ),
    others_read( NotUsedPerm ),
    others_write( NotUsedPerm ),
    others_exec( NotUsedPerm ),
    suid( NotUsedPerm ),
    sgid( NotUsedPerm ),
    sticky( NotUsedPerm ),
    t_regular( NotUsedPerm ),
    t_dir( NotUsedPerm ),
    t_char( NotUsedPerm ),
    t_block( NotUsedPerm ),
    t_symlink( NotUsedPerm ),
    t_fifo( NotUsedPerm ),
    t_socket( NotUsedPerm ),
    size_type( NotUsed ),
    size( 0 ),
    date_type( NotUsed ),
    time( 0 ),
    complementary_names( false ) {
}

FinderQuery::~FinderQuery() {
}

void FinderQuery::setTime( int day, int month, int year ) {
    struct tm m_time;
    m_time.tm_sec = 0;
    m_time.tm_min = 0;
    m_time.tm_hour = 0;
    m_time.tm_mday = day;
    m_time.tm_mon = month;
    m_time.tm_year = year;
    
    time = mktime( &m_time );
}

void FinderQuery::setTime( int day, int month, int year, int hour, int minute, int second ) {
    struct tm m_time;
    m_time.tm_sec = second;
    m_time.tm_min = minute;
    m_time.tm_hour = hour;
    m_time.tm_mday = day;
    m_time.tm_mon = month;
    m_time.tm_year = year;
    
    time = mktime( &m_time );
}

bool FinderQuery::fileMatch( QString file_name ) {
    bool b_return = false;
    for (QValueList<QRegExp>::Iterator it = name_regexp_list.begin() ; it != name_regexp_list.end() ; ++it ) {
        if (pattern_part_match) {
            if ( (*it).search( file_name ) != -1 ) {
                b_return = true;
                break;
            }
        }
        else
            if ( (*it).exactMatch( file_name ) ) {
                b_return = true;
                break;
            }
    }
    return complementary_names ? !b_return : b_return;
}

bool FinderQuery::buildRegExp() {
    name_regexp_list.clear();
    for (QStringList::Iterator it = patterns.begin() ; it != patterns.end() ; ++it ) {
        QString p = *it;
        QRegExp regexp( *it, pattern_case_sensitive, !pattern_regular_expression );
        if ( regexp.isValid() )
            name_regexp_list.append( regexp );
        else
            return false;
    }
    content_regexp = QRegExp( containing, content_case_sensitive, !content_regular_expression );
    if( !content_regexp.isValid() )
        return false;
    return true;
}

bool FinderQuery::contentMatch( const QString line ) {
    return line.find( content_regexp ) != -1;
}
