/***************************************************************************
                          searchquery.cpp  -  description
                             -------------------
    begin                : P j 4 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "searchquery.h"
#include <sys/types.h>
#include <qfile.h>

SearchQuery::SearchQuery( FinderQuery *m_query, QObject *parent, const char *name )
    :QObject( parent, name ), query( m_query ), searching( false ), stop_search( false )
{
}

SearchQuery::~SearchQuery() {
    stop();
}

void SearchQuery::start() {
    stop_search = false;
    searching = true;
    if( query->search_subdir)
        list_job = KIO::listRecursive( query->where_search.first(), false );
    else
        list_job = KIO::listDir( query->where_search.first(), false );
    query->where_search.pop_front();

    connect( list_job, SIGNAL( result( KIO::Job* ) ), SLOT( finished( KIO::Job* ) ) );
    connect( list_job, SIGNAL( entries( KIO::Job*, const KIO::UDSEntryList& ) ), SLOT( listedFiles( KIO::Job*, const KIO::UDSEntryList& ) ) );
}

void SearchQuery::stop() {
    if( searching ) {
        list_job->kill(false);
        searching = false;
        stop_search = true;
    }
}

void SearchQuery::finished( KIO::Job* job) {
    searching = false;
    stop_search = false;
    if (job->error()) {
        job->showErrorDialog();
        emit finished();
        return;
    }
    if (!query->where_search.isEmpty())
        start();
    else
        emit finished();
}

void SearchQuery::listedFiles( KIO::Job *job, const KIO::UDSEntryList& list) {
    KURL base_url = ((KIO::SimpleJob*)job)->url();
    found_files.clear();
    for( KIO::UDSEntryListConstIterator file_entry = list.begin(); file_entry != list.end(); ++file_entry ) {
        QString m_name;
        QString name;
        KURL url = base_url;
        QString user;
        QString group;
        long long size = 0;
        time_t time = 0;
        mode_t type = 0;
        for( KIO::UDSEntry::const_iterator it = (*file_entry).begin(); it !=(*file_entry).end(); ++it ) {

            switch( (*it).m_uds ) {
                case KIO::UDS_SIZE:
                    size = (*it).m_long;
                    break;
                case KIO::UDS_USER:
                    user = (*it).m_str;
                    break;
                case KIO::UDS_GROUP:
                    group = (*it).m_str;
                    break;
                case KIO::UDS_NAME:
                    m_name = (*it).m_str;
                    url.addPath( m_name );
                    name = url.fileName();
                    break;
                case KIO::UDS_MODIFICATION_TIME:
                    time = (*it).m_long;
                    break;
                case KIO::UDS_FILE_TYPE:
                    type = (*it).m_long;
                    break;
                default:
                    break;
            }
        }

        if( name == "." || name == ".." ) continue;

        if( last_dir != url.directory() ) {
            last_dir = url.directory();
            emit currentPath( last_dir );
        }

        if( !query->owner.isNull() && query->owner != user ) continue;
        if( !query->group.isNull() && query->group != group ) continue;
        switch( query->owner_read ) {
            case FinderQuery::NotContain:
                if( type & S_IRUSR ) continue;
            case FinderQuery::Contain:
                if( !(type & S_IRUSR )) continue;
        }
        switch( query->owner_write ) {
            case FinderQuery::NotContain:
                if( type & S_IWUSR ) continue;
            case FinderQuery::Contain:
                if( !(type & S_IWUSR )) continue;
        }
        switch( query->owner_exec ) {
            case FinderQuery::NotContain:
                if( type & S_IXUSR ) continue;
            case FinderQuery::Contain:
                if( !(type & S_IXUSR )) continue;
        }
        switch( query->group_read ) {
            case FinderQuery::NotContain:
                if( type & S_IRGRP ) continue;
            case FinderQuery::Contain:
                if( !(type & S_IRGRP )) continue;
        }
        switch( query->group_write ) {
            case FinderQuery::NotContain:
                if( type & S_IWGRP ) continue;
            case FinderQuery::Contain:
                if( !(type & S_IWGRP )) continue;
        }
        switch( query->group_exec ) {
            case FinderQuery::NotContain:
                if( type & S_IXGRP ) continue;
            case FinderQuery::Contain:
                if( !(type & S_IXGRP )) continue;
        }
        switch( query->others_read ) {
            case FinderQuery::NotContain:
                if( type & S_IROTH ) continue;
            case FinderQuery::Contain:
                if( !(type & S_IROTH )) continue;
        }
        switch( query->others_write ) {
            case FinderQuery::NotContain:
                if( type & S_IWOTH ) continue;
            case FinderQuery::Contain:
                if( !(type & S_IWOTH )) continue;
        }
        switch( query->others_exec ) {
            case FinderQuery::NotContain:
                if( type & S_IXOTH ) continue;
            case FinderQuery::Contain:
                if( !(type & S_IXOTH )) continue;
        }
        switch( query->suid ) {
            case FinderQuery::NotContain:
                if( type & S_ISUID ) continue;
            case FinderQuery::Contain:
                if( !(type & S_ISUID )) continue;
        }
        switch( query->sgid ) {
            case FinderQuery::NotContain:
                if( type & S_ISGID ) continue;
            case FinderQuery::Contain:
                if( !(type & S_ISGID )) continue;
        }
        switch( query->sticky ) {
            case FinderQuery::NotContain:
                if( type & S_ISVTX ) continue;
            case FinderQuery::Contain:
                if( !(type & S_ISVTX )) continue;
        }
        switch( query->t_regular ) {
            case FinderQuery::NotContain:
                if( S_ISREG(type) ) continue;
            case FinderQuery::Contain:
                if( !S_ISREG(type) ) continue;
        }
        switch( query->t_dir ) {
            case FinderQuery::NotContain:
                if( S_ISDIR(type) ) continue;
            case FinderQuery::Contain:
                if( !S_ISDIR(type) ) continue;
        }
        switch( query->t_char ) {
            case FinderQuery::NotContain:
                if( S_ISCHR(type) ) continue;
            case FinderQuery::Contain:
                if( !S_ISCHR(type) ) continue;
        }
        switch( query->t_block ) {
            case FinderQuery::NotContain:
                if( S_ISBLK(type) ) continue;
            case FinderQuery::Contain:
                if( !S_ISBLK(type) ) continue;
        }
        switch( query->t_symlink ) {
            case FinderQuery::NotContain:
                if( S_ISLNK(type) ) continue;
            case FinderQuery::Contain:
                if( !S_ISLNK(type) ) continue;
        }
        switch( query->t_fifo ) {
            case FinderQuery::NotContain:
                if( S_ISFIFO(type) ) continue;
            case FinderQuery::Contain:
                if( !S_ISFIFO(type) ) continue;
        }
        switch( query->t_socket ) {
            case FinderQuery::NotContain:
                if( S_ISSOCK(type) ) continue;
            case FinderQuery::Contain:
                if( !S_ISSOCK(type) ) continue;
        }
        switch( query->date_type ) {
            case FinderQuery::More:
                if( !(time > query->time) ) continue;
            case FinderQuery::MoreEq:
                if( !(time >= query->time) ) continue;
            case FinderQuery::Eq:
                if( !(time = query->time) ) continue;
            case FinderQuery::LessEq:
                if( !(time <= query->time) ) continue;
            case FinderQuery::Less:
                if( !(time < query->time) ) continue;
        }
        switch( query->size_type ) {
            case FinderQuery::More:
                if( !(size > query->size) ) continue;
            case FinderQuery::MoreEq:
                if( !(size >= query->size) ) continue;
            case FinderQuery::Eq:
                if( !(size = query->size) ) continue;
            case FinderQuery::LessEq:
                if( !(size <= query->size) ) continue;
            case FinderQuery::Less:
                if( !(size < query->size) ) continue;
        }
        if (!query->fileMatch( name )) continue;

        if (query->search_content && !searchContent( url.path() ) ) continue;

        //files that reached here are the searched one
        found_files.append( new KFileItem( *file_entry, base_url, false, true ) );
    }
    if( !found_files.isEmpty() )
        emit( foundFiles( found_files ) );
}

void SearchQuery::slotCurrentPath( const KURL& url ) {
    emit currentPath( url.prettyURL() );
}

bool SearchQuery::searchContent( const QString file ) {
    QFile f(file);

    if( f.open(IO_ReadOnly) ) {
        QTextStream text(&f);
        text.setEncoding(QTextStream::Locale);
        QString line;
        while(!stop_search && !text.atEnd() ){
            line = text.readLine();
            if (query->contentMatch(line)) {
                f.close();
                return true;
            }
        }
        f.close();
    }
    return false;
}
