#include "plistview.h"

PListView::PListView( QWidget *parent, const char *name )
  : KListView( parent, name ) {
//	setSelectionModeExt( KListView::Single );
}

PListView::~PListView() {
}

void PListView::mselectAll( bool select ) {
	for( PListViewItem *item=(PListViewItem*)firstChild(); item; item=(PListViewItem*)item->itemBelow() )
		item->setmSelected( select );
	update();
}

void PListView::contentsMousePressEvent( QMouseEvent *e ) {
    PListViewItem * new_item = (PListViewItem*)itemAt( contentsToViewport( e->pos() ));
    PListViewItem * old_item = (PListViewItem*)currentItem();
    
    if (e->state() & ControlButton)
	if (new_item) new_item->setmSelected(!new_item->ismSelected() );
	else;
    else 
	if (e->state() & ShiftButton) {
	    if (!old_item) {
		if (!new_item) return;
		new_item->setmSelected( true );
	    }
	    else {
		PListViewItem *top; //of the new selection
		PListViewItem *bottom;
		if ( old_item->itemPos() < e->y() )
		    if (!new_item) {
			top = old_item;
			bottom = (PListViewItem*)lastItem();
		    }
		    else
			top = old_item, bottom = new_item;
		else 
		    if (!new_item) {
			top = (PListViewItem*)firstChild();
			bottom = old_item;
		    }
		    else
			top = new_item, bottom = old_item;
		for ( ; top && top != bottom ; top = (PListViewItem*)top->nextSibling() )
		    top->setmSelected( true );
		if ( bottom ) bottom->setmSelected( true );
	    }
	}

    KListView::contentsMousePressEvent(e);
}

void PListView::keyPressEvent (QKeyEvent* e) {
	//don't care whether it's on the keypad or not
	int e_state=(e->state() & ~Keypad);

	PListViewItem* item = (PListViewItem*)currentItem();
	if (item==0) return;

	setUpdatesEnabled(false);

	if (e->key() != Key_Escape)
		KListView::keyPressEvent (e);
	
	PListViewItem* current = (PListViewItem*)currentItem();
	PListViewItem* nextItem = 0L;

	bool shiftOrCtrl((e_state==ControlButton) || (e_state==ShiftButton));
	switch (e->key()) {
		case Key_Escape:
			PListView::selectAll(false);
			break;

		case Key_Space:
			//toggle selection of current item
			item->setmSelected(!item->ismSelected());
//			item->repaint();
			break;

		case Key_Insert:
			//toggle selection of current item and move to the next item
			item->setmSelected(!item->ismSelected());
//			item->repaint();
			nextItem=(PListViewItem*)item->itemBelow();

			if (nextItem!=0) {
				setCurrentItem(nextItem);
				ensureItemVisible(nextItem);
			};
			break;

		case Key_Down:
			//toggle selection of current item and move to the next item
			if (shiftOrCtrl)
				item->setmSelected(!item->ismSelected());
			break;

		case Key_Up:
			//move to the prev. item and toggle selection of this one
			if (shiftOrCtrl)
				item->setmSelected(!item->ismSelected());
			break;

		case Key_End:
			//move to the last item and toggle selection of all items inbetween
			nextItem=item;

			while(nextItem!=0) {
				if (shiftOrCtrl)
					nextItem->setmSelected(!nextItem->ismSelected());
				nextItem=(PListViewItem*)nextItem->itemBelow();
			}
			break;

		case Key_Home:
			//move to the last item and toggle selection of all items inbetween
			nextItem=item;

			while(nextItem!=0) {
				if (shiftOrCtrl)
					nextItem->setmSelected(!nextItem->ismSelected());
				nextItem=(PListViewItem*)nextItem->itemAbove();
			}
			break;

		case Key_Next:
			nextItem=item;

			if (shiftOrCtrl)
				for (nextItem=item; nextItem != current; nextItem=(PListViewItem*)nextItem->itemBelow())
					nextItem->setmSelected(!nextItem->ismSelected());
			break;

		case Key_Prior:
			nextItem=item;

			if (shiftOrCtrl)
				for (nextItem=item; nextItem != current; nextItem=(PListViewItem*)nextItem->itemAbove())
					nextItem->setmSelected(!nextItem->ismSelected());
			break;
		default:
			break;
	}

	setUpdatesEnabled(true);
	update();
}

void PListView::focusInEvent( QFocusEvent *fe ) {
	KListView::focusInEvent( fe );
	if ( currentItem() )
		currentItem()->setSelected(true);
}

void PListView::focusOutEvent( QFocusEvent *fe ) {
/*	if ( currentItem() ) {
		currentItem()->setSelected(false);
		currentItem()->repaint();
	}*/
	KListView::focusOutEvent( fe );
}

//
//------------- PListViewItem --------------------
//
PListViewItem::PListViewItem(QListView *parent)
  : KListViewItem(parent) {
	init();
}

PListViewItem::PListViewItem(QListViewItem *parent)
  : KListViewItem(parent) {
	init();
}

PListViewItem::PListViewItem(QListView *parent, QListViewItem *after)
  : KListViewItem(parent, after) {
	init();
}

PListViewItem::PListViewItem(QListViewItem *parent, QListViewItem *after)
  : KListViewItem(parent, after) {
	init();
}

PListViewItem::PListViewItem(QListView *parent,
    QString label1, QString label2, QString label3, QString label4,
    QString label5, QString label6, QString label7, QString label8)
  : KListViewItem(parent, label1, label2, label3, label4, label5, label6, label7, label8) {
	init();
}

PListViewItem::PListViewItem(QListViewItem *parent,
    QString label1, QString label2, QString label3, QString label4,
    QString label5, QString label6, QString label7, QString label8)
  : KListViewItem(parent, label1, label2, label3, label4, label5, label6, label7, label8) {
	init();
}

PListViewItem::PListViewItem(QListView *parent, QListViewItem *after,
    QString label1, QString label2, QString label3, QString label4,
    QString label5, QString label6, QString label7, QString label8)
  : KListViewItem(parent, after, label1, label2, label3, label4, label5, label6, label7, label8) {
	init();
}

PListViewItem::PListViewItem(QListViewItem *parent, QListViewItem *after,
    QString label1, QString label2, QString label3, QString label4,
    QString label5, QString label6, QString label7, QString label8)
  : KListViewItem(parent, after, label1, label2, label3, label4, label5, label6, label7, label8) { init();
}

PListViewItem::~PListViewItem() {
}

void PListViewItem::init() {
  is_selected = false;
}

bool PListViewItem::ismSelected() const {
	return is_selected;
}

void PListViewItem::setmSelected( bool sel ) {
	if (sel != is_selected) {
		is_selected = sel;
		emit ((PListView*)listView())->mSelectionChanged();
		repaint();
	}
}

void PListViewItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int alignment) {
	QColorGroup _cg = cg;
	if (ismSelected()) {
		_cg.setColor(QColorGroup::Text, QColor("red"));
		_cg.setColor(QColorGroup::HighlightedText, QColor("red"));
	}

	KListViewItem::paintCell(p, _cg, column, width, alignment);
}
