/***************************************************************************
                          plistview.h  -  description
                             -------------------
    begin                : Mon May 6 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PLISTVIEW_H
#define PLISTVIEW_H

#include <klistview.h>

class PListViewItem;

/**
 * @short Implements special selection of @ref PListViewItem.
 * 
 * Parent of PFileView. Implements special selection of @ref PListViewItem.
 */
class PListView : public KListView {
    Q_OBJECT
    friend PListViewItem;
public:
    /**
     * Constructor.
     *
     * The parameters @p parent and @p name are handled by
     * @ref QListView, as usual.
     */
    PListView (QWidget *parent = 0, const char *name = 0);

    /**
     * Destructor.
     */
    virtual ~PListView();

signals:
    void mSelectionChanged();

public slots:
    void mselectAll( bool select );

protected:
    virtual void keyPressEvent (QKeyEvent*);

    virtual void contentsMousePressEvent (QMouseEvent *);

    virtual void focusInEvent( QFocusEvent*);
    virtual void focusOutEvent( QFocusEvent*);
};

/**
* A listview item with support for alternate background colours. It is
* a drop-in replacement for @ref QListViewItem
*
* @short listview item with alternate background colour support
*/
class PListViewItem : public KListViewItem {
public:
    /**
     * constructors. The semantics remain as in @ref QListViewItem.
     * Although they accept a @ref QListViewItem as parent, please
     * don't mix PListViewItem (or subclasses) with QListViewItem
     * (or subclasses).
     */
    PListViewItem(QListView *parent);
    PListViewItem(QListViewItem *parent);
    PListViewItem(QListView *parent, QListViewItem *after);
    PListViewItem(QListViewItem *parent, QListViewItem *after);

    PListViewItem(QListView *parent,
      QString, QString = QString::null,
      QString = QString::null, QString = QString::null,
      QString = QString::null, QString = QString::null,
      QString = QString::null, QString = QString::null);

    PListViewItem(QListViewItem *parent,
      QString, QString = QString::null,
      QString = QString::null, QString = QString::null,
      QString = QString::null, QString = QString::null,
      QString = QString::null, QString = QString::null);

    PListViewItem(QListView *parent, QListViewItem *after,
      QString, QString = QString::null,
      QString = QString::null, QString = QString::null,
      QString = QString::null, QString = QString::null,
      QString = QString::null, QString = QString::null);

    PListViewItem(QListViewItem *parent, QListViewItem *after,
      QString, QString = QString::null,
      QString = QString::null, QString = QString::null,
      QString = QString::null, QString = QString::null,
      QString = QString::null, QString = QString::null);

    virtual ~PListViewItem();

    bool ismSelected() const;
    virtual void setmSelected( bool sel );

private:
    void init();
    void paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int alignment);

    bool is_selected;
};

#endif
