#include "purllabel.h"
#include <kglobalsettings.h>

PURLLabel::PURLLabel( QWidget * parent, const char * name )
: QLabel( parent, name ), working( false ), highlighted_link_color( Qt::red ) {
	setMouseTracking( true );
	setTextFormat( Qt::RichText );
	connect( this, SIGNAL( currentURLChanged( const QString& ) ), SLOT( changeText() ) );
}

PURLLabel::~PURLLabel() {
}

void PURLLabel::setURL( const QString& url ) {
	whole_url = url;
	current_url = QString::null;
	rest_of_url = whole_url;
	changeText();
}

void PURLLabel::mouseReleaseEvent (QMouseEvent* e) {
	QLabel::mouseReleaseEvent (e);
	switch (e->button()) {
		case LeftButton:
			emit leftClickedURL ();
			emit leftClickedURL (current_url);
			break;

		case MidButton:
			emit middleClickedURL ();
			emit middleClickedURL (current_url);
			break;

		case RightButton:
			emit rightClickedURL ();
			emit rightClickedURL (current_url);
			break;
		  
		default:
			; // nothing
	}
}

void PURLLabel::mouseMoveEvent ( QMouseEvent *e ) {
	if (!working) {
		working = true;
		old_current_url = current_url;
		QPoint p = e->pos();
		QFontMetrics fm = fontMetrics();
		if (p.x() > fm.width( whole_url ) ) {
			current_url = whole_url;
			rest_of_url = QString::null;
		}
		else {
			int i;
			for ( i = 0; p.x() > fm.width( whole_url.section( '/', 0, i, QString::SectionIncludeTrailingSep ) ); i++ );
			current_url = whole_url.section( '/', 0, i, QString::SectionIncludeTrailingSep );
			rest_of_url = whole_url.section( '/', i+1 );
		}

		if ( old_current_url != current_url )
			emit currentURLChanged( current_url );
		working = false;
	}
}

void PURLLabel::enterEvent ( QEvent *e ) {
	QLabel::enterEvent (e);

	old_current_url = QString::null;
	current_url = QString::null;
	rest_of_url = QString::null;
}

void PURLLabel::leaveEvent ( QEvent *e ) {
	QLabel::leaveEvent (e);

	old_current_url = QString::null;
	current_url = QString::null;
	rest_of_url = whole_url;
	changeText();
}

void PURLLabel::changeText() {
	setText( QString( "<font color=\"%1\">%2</font><font color=\"%3\">%4</font>" ).arg( highlighted_link_color.name() )
			.arg( current_url )
			.arg( link_color.name() )
			.arg( rest_of_url ) );
}

void PURLLabel::focusInParent () {
	link_color = QColor( palette().active().highlightedText() );
	changeText();
}

void PURLLabel::focusOutParent () {
	link_color = KGlobalSettings::linkColor();
	changeText();
}
