/***************************************************************************
                          gourldialog.cpp  -  description
                             -------------------
    begin                : Fri Apr 5 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlayout.h>
#include <klocale.h>
#include <qlabel.h>
#include <qvbox.h>
#include <kurlcompletion.h>
#include <klistview.h>
#include <qpushbutton.h>
#include <kmessagebox.h>
#include <qlineedit.h>
#include <qcheckbox.h>

#include "gourldialog.h"
#include "../pcmm.h"

class PURLViewItem: public KListViewItem {
public:
    PURLViewItem( KListView *view, PURLItem *item_ )
    : KListViewItem( view, item_->name() ), item(item_) {
    }

    PURLViewItem( KListView *view, const QString& protocol, PURLItem *item_ )
    : KListViewItem( view, protocol, item_->name() ), item(item_) {
    }

    PURLViewItem( KListView *view, bool bookmark, PURLItem *item_ )
    : KListViewItem( view, bookmark ? bookmark_label : QString::null, item_->name() ), item(item_) {
    }

    PURLViewItem( KListView *view, bool bookmark, const QString& protocol, PURLItem *item_ )
    : KListViewItem( view, bookmark ? bookmark_label : QString::null, protocol, item_->name() ), item(item_) {
    }

    ~PURLViewItem() {
    }

    PURLItem *urlItem() {
            return item;
    }

private:
    PURLItem *item;
    static QString bookmark_label;
};

QString PURLViewItem::bookmark_label = "B";

GoURLDialog::GoURLDialog(QWidget *parent, const QString& caption )
: KDialogBase(Tabbed, caption, Ok|Cancel|Default, Ok, parent, "GoURLDialog", true ) {
    setButtonText( Default, i18n("&Database actions") );
    setButtonText( Ok, i18n("&Go to URL") );
    actions = new KPopupMenu( this, "actionspopupmenu" );
    insert_id = actions->insertItem( i18n("Insert..."), this, SLOT( insertURL() ) );
    edit_id = actions->insertItem( i18n("Edit..."), this, SLOT( editURL() ) );
    remove_id = actions->insertItem( i18n( "Remove" ), this, SLOT( removeURL() ) );
    actionButton( Default )->setPopup( actions );
    connect( actions, SIGNAL( aboutToShow() ), this, SLOT( updateActions() ) );
	
    tab_goto = addPage( i18n("Go") );
    QVBoxLayout *go_layout = new QVBoxLayout( tab_goto );
    QHBoxLayout *combo_layout = new QHBoxLayout( go_layout );
    QLabel *label = new QLabel( i18n("Insert URL:"), tab_goto );
    path_combobox = new KURLComboBox( KURLComboBox::Directories, true, tab_goto, "pathcombobox" );
    combo_layout->addWidget( label );
    combo_layout->addWidget( path_combobox );
    go_layout->addStretch();

    path_combobox->setFocus();
    path_combobox->setMinimumWidth( 500 );
    KURLCompletion *completion = new KURLCompletion( KURLCompletion::DirCompletion );
    completion->setIgnoreCase( true );
    path_combobox->setCompletionObject( completion );
    connect( path_combobox, SIGNAL( returnPressed() ), this, SLOT( accept() ) );
    connect( path_combobox, SIGNAL( textChanged( const QString& ) ), this, SLOT( setCaption( const QString& ) ) );

    tab_bookmark = addVBoxPage( i18n( "Bookmarks" ) );
    bookmarks_view = new KListView( tab_bookmark, "url_list" );
    bookmarks_view->addColumn( i18n( "Protocol" ) );
    bookmarks_view->addColumn( i18n( "Url" ) );
    bookmarks_view->setAllColumnsShowFocus( true );
    connect( bookmarks_view, SIGNAL( currentChanged ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );
    connect( bookmarks_view, SIGNAL( clicked ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );
    connect( bookmarks_view, SIGNAL( spacePressed ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );

    tab_all = addVBoxPage( i18n( "All" ));
    all_view = new KListView( tab_all, "url_list" );
    all_view->addColumn( QString::null );
    all_view->addColumn( i18n( "Protocol" ) );
    all_view->addColumn( i18n( "Url" ) );
    all_view->setAllColumnsShowFocus( true );
    connect( all_view, SIGNAL( currentChanged ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );
    connect( all_view, SIGNAL( clicked ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );
    connect( all_view, SIGNAL( spacePressed ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );

    tab_ftp = addVBoxPage( i18n( "FTP" ));
    ftp_view = new KListView( tab_ftp, "url_list" );
    ftp_view->addColumn( QString::null );
    ftp_view->addColumn( i18n( "Url" ) );
    ftp_view->setAllColumnsShowFocus( true );
    connect( ftp_view, SIGNAL( currentChanged ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );
    connect( ftp_view, SIGNAL( clicked ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );
    connect( ftp_view, SIGNAL( spacePressed ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );

    tab_smb = addVBoxPage( i18n( "Samba" ));
    smb_view = new KListView( tab_smb, "url_list" );
    smb_view->addColumn( QString::null );
    smb_view->addColumn( i18n( "Url" ) );
    smb_view->setAllColumnsShowFocus( true );
    connect( smb_view, SIGNAL( currentChanged ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );
    connect( smb_view, SIGNAL( clicked ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );
    connect( smb_view, SIGNAL( spacePressed ( QListViewItem * ) ), SLOT( changeText( QListViewItem * ) ) );

    actualizeURLs();
}

GoURLDialog::~GoURLDialog(){
}

void GoURLDialog::setCurrentDirectory( KURL& current_dir ) {
    path_combobox->setEditURL( current_dir );
}

void GoURLDialog::setGotoTabActive() {
    showPage( 0 );
}

void GoURLDialog::setBookmarksTabActive() {
    showPage( pageIndex( tab_bookmark ) );
}

void GoURLDialog::actualizeURLs() {
    ftp_view->clear();
    smb_view->clear();
    all_view->clear();
    bookmarks_view->clear();
    PURLItemList::iterator it;

    PURLItemList *ftp_list = Pcmm::url_database->ftpList();
    if (ftp_list)
        for( it = ftp_list->begin() ; it != ftp_list->end() ; ++it )
            new PURLViewItem( ftp_view, (*it).isBookmark(), &*it );

    PURLItemList *smb_list = Pcmm::url_database->sambaList();
    if (smb_list)
        for( it = smb_list->begin() ; it != smb_list->end() ; ++it )
            new PURLViewItem( smb_view, (*it).isBookmark(), &*it );

    QDictIterator<PURLItemList> protocol_it( Pcmm::url_database->database() );
    for( ; protocol_it.current() ; ++protocol_it )
        for( it = (*protocol_it)->begin() ; it != (*protocol_it)->end() ; ++it ) {
            new PURLViewItem( all_view, (*it).isBookmark(), protocol_it.currentKey(), &*it );
            if ((*it).isBookmark())
                new PURLViewItem( bookmarks_view, protocol_it.currentKey(), &*it );
        }
}

void GoURLDialog::updateActions() {
    actions->setItemEnabled( edit_id, false );
    actions->setItemEnabled( remove_id, false );
    bool enable = false;
    switch ( activePageIndex() ) {
        case 1: //tab_all
            if ( (PURLViewItem*)bookmarks_view->currentItem() ) enable = true;
            break;
        case 2: //tab_bookmark
            if ( (PURLViewItem*)all_view->currentItem() ) enable = true;
            break;
        case 3: //tab_ftp
            if ( (PURLViewItem*)ftp_view->currentItem() ) enable = true;
            break;
        case 4: //tab_smb
            if ( (PURLViewItem*)smb_view->currentItem() ) enable = true;
            break;
    }
    if (enable) {
        actions->setItemEnabled( edit_id, true );
        actions->setItemEnabled( remove_id, true );
    }
}

void GoURLDialog::changeText( QListViewItem *item ) {
if (item)
    path_combobox->setEditURL( ((PURLViewItem*)item)->urlItem()->url().prettyURL() );
}

void GoURLDialog::insertURL() {
    KDialogBase dlg( KDialogBase::Plain, i18n("Insert URL"), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, this, "inserturldialog" );
    QFrame *widget = dlg.plainPage();
    QGridLayout *l = new QGridLayout( widget, 2, 2 );
    
    QLabel *label1 = new QLabel( i18n("URL"), widget );
    l->addWidget( label1, 0, 0 );

    QLabel *label2 = new QLabel( i18n("Name"), widget );
    l->addWidget( label2, 1, 0 );

    QString content = path_combobox->currentText();

    QLineEdit *edit1 = new QLineEdit( content, widget );
    edit1->setMinimumWidth( 300 );
    edit1->setFocus();
    l->addWidget( edit1, 0, 1 );

    QLineEdit *edit2 = new QLineEdit( content, widget );
    l->addWidget( edit2, 1, 1 );

    QCheckBox *bookmark = new QCheckBox( i18n("Set as bookmark"), widget );
    if (activePageIndex() == pageIndex( tab_bookmark ) ) bookmark->setChecked( true );
    l->addWidget( bookmark, 2, 1 );

    if ( dlg.exec() ) {
        KURL url( edit1->text() );
        if ( url.isValid() ) {
            PURLItem item( url, edit2->text(), bookmark->isChecked() );
            Pcmm::url_database->insert( item, false );
            actualizeURLs();
        }
        else
            KMessageBox::sorry( this, i18n("URL is not valid.") );
    }
}

void GoURLDialog::editURL() {
    PURLViewItem *item = 0;
    bool edit = false;
    switch ( activePageIndex() ) {
        case 1: //tab_bookmark
            if ( item = (PURLViewItem*)bookmarks_view->currentItem() ) edit = true;
            break;
        case 2: //tab_all
            if ( item = (PURLViewItem*)all_view->currentItem() ) edit = true;
            break;
        case 3: //tab_ftp
            if ( item = (PURLViewItem*)ftp_view->currentItem() ) edit = true;
            break;
        case 4: //tab_smb
            if ( item = (PURLViewItem*)smb_view->currentItem() ) edit = true;
            break;
    }
    if (edit) {
        KDialogBase dlg( KDialogBase::Plain, i18n("Insert URL"), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, this, "inserturldialog" );
        QFrame *widget = dlg.plainPage();
        QGridLayout *l = new QGridLayout( widget, 2, 2 );
        
        QLabel *label1 = new QLabel( i18n("URL"), widget );
        l->addWidget( label1, 0, 0 );

        QLabel *label2 = new QLabel( i18n("Name"), widget );
        l->addWidget( label2, 1, 0 );

        QLineEdit *edit1 = new QLineEdit( widget );
        edit1->setMinimumWidth( 300 );
        edit1->setFocus();
        l->addWidget( edit1, 0, 1 );

        QLineEdit *edit2 = new QLineEdit( widget );
        l->addWidget( edit2, 1, 1 );

        QCheckBox *bookmark = new QCheckBox( i18n("Set as bookmark"), widget );
        l->addWidget( bookmark, 2, 1 );

        PURLItem *url_item = item->urlItem();
        edit1->setText( url_item->url().prettyURL() );
        edit2->setText( url_item->name() );
        bookmark->setChecked( url_item->isBookmark() );

        if ( dlg.exec() ) {
            KURL url( edit1->text() );
            if ( url.isValid() ) {
                url_item->setURL( url );
                url_item->setName( edit2->text() );
                url_item->setBookmark( bookmark->isChecked() );
                actualizeURLs();
            }
            else
                KMessageBox::sorry( this, i18n("URL can't be changed. It is not valid.") );
        }
    }
}

void GoURLDialog::removeURL() {
    PURLViewItem *item = 0;
    bool remove = false;
    switch ( activePageIndex() ) {
        case 1: //tab_bookmark
            if ( item = (PURLViewItem*)bookmarks_view->currentItem() ) remove = true;
            break;
        case 2: //tab_all
            if ( item = (PURLViewItem*)all_view->currentItem() ) remove = true;
            break;
        case 3: //tab_ftp
            if ( item = (PURLViewItem*)ftp_view->currentItem() ) remove = true;
            break;
        case 4: //tab_smb
            if ( item = (PURLViewItem*)smb_view->currentItem() ) remove = true;
            break;
    }
    if (remove) {
        Pcmm::url_database->remove( *item->urlItem() );
        actualizeURLs();
    }
}
