/***************************************************************************
                          gourldialog.h  -  description
                             -------------------
    begin                : Fri Apr 5 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GOURLDIALOG_H
#define GOURLDIALOG_H

#include <qwidget.h>
#include <kdialogbase.h>
#include <kcombobox.h>
#include <klocale.h>
#include <kurlcombobox.h>
#include <kpopupmenu.h>

#include "purldatabase.h"

/**
 * @short Dialog that lets you go to a specified location
 *
 * This dialog consist of a combobox where you can specify a location you want to go.
 * It is also connected with database that saves various paths. You can watch this on
 * several tabs.
 * @see PURLDatabase
 * 
 *@author Tom Potrusil
 */

class GoURLDialog : public KDialogBase  {
   Q_OBJECT
public: 
   /**
    * @param parent parent of the dialog
    * @param caption caption of the dialog
    * @param database pointer to a database
    */
    GoURLDialog(QWidget *parent, const QString& caption );
    ~GoURLDialog();

    /**
     * Sets go destination to current_dir
     */
    void setCurrentDirectory( KURL& current_dir );

    /**
     * Set "goto tab" active.
     */
    void setGotoTabActive();

    /**
     * Set "bookmarks tab" active.
     */
    void setBookmarksTabActive();

    /**
     * Synchonize dialog with database
     */
    void actualizeURLs();

    KURLComboBox *path_combobox;

protected slots:
    void updateActions();

private slots:
    void changeText( QListViewItem *item );
    void insertURL();
    void editURL();
    void removeURL();

private:
    QFrame *tab_goto;
    QVBox *tab_bookmark;
    QVBox *tab_all;
    QVBox *tab_ftp;
    QVBox *tab_smb;
    KListView *ftp_view;
    KListView *smb_view;
    KListView *all_view;
    KListView *bookmarks_view;

    int insert_id, edit_id, remove_id;
    KPopupMenu *actions;
};

#endif
