/***************************************************************************
                          purldatabase.cpp  -  description
                             -------------------
    begin                : Mon May 13 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "purldatabase.h"
#include "../pcmm_general.h"
#include <kstandarddirs.h>
#include <kglobal.h>
#include <qfile.h>

PURLDatabase::PURLDatabase(){
    load();
}

PURLDatabase::~PURLDatabase(){
    save();
}

void PURLDatabase::insert( PURLItem item, bool check_for_same ) {
    PURLItemList *list = checkProtocol( item );
    if (!check_for_same || ( list->find( item ) == list->end() ) )
        list->append( item );
}

void PURLDatabase::remove( PURLItem& item ) {
    PURLItemList *list;
    if ( list = checkProtocol( item, false ) );
        list->remove( list->find( item ) );
}

PURLItemList *PURLDatabase::sambaList() {
    return database().find( "smb" );
}

PURLItemList *PURLDatabase::ftpList() {
    return database().find( "ftp" );
}

PURLItemPtrList& PURLDatabase::bookmarkList() {
    QDictIterator<PURLItemList> protocol_it( url_database );
    PURLItemList::iterator it;
    m_list.clear();
    for( ; protocol_it.current() ; ++protocol_it )
        for( it = (*protocol_it)->begin() ; it != (*protocol_it)->end() ; ++it )
            if ( (*it).isBookmark() )
                m_list.append( &(*it) );
    return m_list;
}

PURLItemList *PURLDatabase::checkProtocol( PURLItem& item, bool insert_not_exist ) {
	PURLItemList *list;
	QString p = item.url().protocol();
	if ( !(list = database().find( p ) ) && insert_not_exist ) {
            list = new PURLItemList;
            database().insert( p, list );
	}
	return list;
}

QDict<PURLItemList>& PURLDatabase::database() {
	return url_database;
}

void PURLDatabase::load() {
    kapp->config()->setGroup( GENERAL_KEY );
    QString database_path = kapp->config()->readEntry( "Database file", QString::null );
    QFile f( database_path );
    if (f.open( IO_ReadOnly ) ) {
	QTextStream s( &f );
	QString line;
	while ( !s.eof() ) {
	    line = s.readLine();
            QString bookmark = line.section(';',0, 0);
	    QString url = line.section(';', 2 );
	    QString name = line.section(';', 1, 1);
	    insert( PURLItem( KURL(url), name, bookmark == "b" ), false );
	}
	f.close();
    }
}

void PURLDatabase::save() {
    kapp->config()->setGroup( GENERAL_KEY );
    QString database_path = kapp->config()->readEntry( "Database file", QString::null );
    if (database_path.isEmpty()) {
        KStandardDirs *dirs = KGlobal::dirs();
        database_path = dirs->saveLocation( "appdata" ) + "urldatabase";
        kapp->config()->writeEntry( "Database file", database_path );
    }
    QFile f( database_path );
    if (f.open( IO_WriteOnly ) ) {
        QTextStream s( &f );

        QDictIterator<PURLItemList> protocol_it( url_database );
        PURLItemList::iterator it;
        for( ; protocol_it.current() ; ++protocol_it )
                for( it = (*protocol_it)->begin() ; it != (*protocol_it)->end() ; ++it )
                        s << ((*it).isBookmark() ? "b;" : ";") << (*it).name() << ";" << (*it).url().prettyURL() << endl;

        f.close();
    }
}
