/***************************************************************************
                          purldatabase.h  -  description
                             -------------------
    begin                : Mon May 13 2002
    copyright            : (C) 2002 by Tom Potrusil
    email                : potrto@centrum.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PURLDATABASE_H
#define PURLDATABASE_H

#include "purlitem.h"
#include <qdict.h>

/* 
 * @short Class that stores URLs
 *
 * Class that stores various URLs that
 * come through application. URLs are
 * devided into several types, such as
 * Samba URLs, FTP URLs ...
 * @see PURLItem
 *
 * @author Tom Potrusil
 */

class PURLDatabase {
public: 
	PURLDatabase();
	~PURLDatabase();

	/**
	 * Inserts @p item into database
         * @param check_for_same before @p item is inserted it is checked if there
         * is no similar one in the database
	 */
	void insert( PURLItem item, bool check_for_same = true );
	
	/* 
	 * Removes @p item from database
	 */
	void remove( PURLItem& item );

	/**
	 * Check the protocol of the @p item
	 * @param item an item of which you want to check protocol
	 * @param insert_not_exist if a list with item's protocol should be created 
	 * when it doesn't exist
	 * 
	 * @returns A protocol list in which the @p item should be inserted
	 */
	PURLItemList *checkProtocol( PURLItem& item, bool insert_not_exist = true );

	/**
	 * @returns bookmark list of pointers to @ref PURLItem
	 */
	PURLItemPtrList& bookmarkList();

	/**
	 * @returns list of @ref PURLItem that have smb protocol
	 */
	PURLItemList *sambaList();

	/**
	 * @returns list of @ref PURLItem that have ftp protocol
	 */
	PURLItemList *ftpList();

	/**
	 * @returns whole database
	 */
	QDict<PURLItemList>& database();

	/**
	 * Read database from a file
	 */
	void load();

	/**
	 * Write database to a disk
	 */
	void save();

private:
	QDict<PURLItemList> url_database;
        PURLItemPtrList m_list;
};

#endif
